﻿#region License Information
/* HeuristicLab
 * Copyright (C) 2002-2019 Heuristic and Evolutionary Algorithms Laboratory (HEAL)
 *
 * This file is part of HeuristicLab.
 *
 * HeuristicLab is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * HeuristicLab is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with HeuristicLab. If not, see <http://www.gnu.org/licenses/>.
 */
#endregion

using HEAL.Attic;
using HeuristicLab.Common;
using HeuristicLab.Core;
using HeuristicLab.Encodings.SymbolicExpressionTreeEncoding;
using HeuristicLab.Random;
using System.Collections.Generic;


namespace HeuristicLab.Problems.DataAnalysis.Symbolic {
  [StorableType("38C37AED-5B9D-488E-8785-D8FF541D9B4C")]
  public sealed class TreeModelTreeNode : SymbolicExpressionTreeTerminalNode {
    public new TreeModel Symbol {
      get { return (TreeModel)base.Symbol; }
    }
    [Storable]
    ISymbolicExpressionTree tree;
    public ISymbolicExpressionTree Tree {
      get { return tree; }
      set { tree = value; }
    }
    int clusterNumer = -10;
    int treeNumber = 0;
    public int ClusterNumer {
      get { return clusterNumer; }
      set { clusterNumer = value; }
    }
    public int TreeNumber {
      get { return treeNumber; }
      set { treeNumber = value; }
    }

    [StorableConstructor]
    private TreeModelTreeNode(StorableConstructorFlag _) : base(_) {
      ClusterNumer = -10;
    }

    private TreeModelTreeNode(TreeModelTreeNode original, Cloner cloner)
      : base(original, cloner) {
      tree = original.tree;
      ClusterNumer = original.ClusterNumer;
      TreeNumber = original.TreeNumber;
    }

    private TreeModelTreeNode() : base() { }
    public TreeModelTreeNode(TreeModel treeModelSymbol) : base(treeModelSymbol) {
      ClusterNumer = -10;
    }

    public override bool HasLocalParameters {
      get { return true; }
    }
    //public override void ResetLocalParameters(IRandom random, List<ISymbolicExpressionTree> modelSet, List<List<int>> map, List<int> cLusterNumber) {
    //  base.ResetLocalParameters(random);
    //  if (ClusterNumer < 0) {
    //    //Tree = map.NewModelForInizializtion(ClusterNumer, random);
    //    var selectedNodelNumber = random.Next(modelSet.Count);
    //    ClusterNumer = cLusterNumber[selectedNodelNumber];
    //    TreeNumber = selectedNodelNumber;
    //    Tree = (ISymbolicExpressionTree)modelSet[selectedNodelNumber].Clone();
    //  } else {
    //    //New Model For Mutation Adding
    //    var selectedNodelNumber = random.Next(map[ClusterNumer].Count);
    //    TreeNumber = selectedNodelNumber;
    //    Tree = (ISymbolicExpressionTree)modelSet[map[ClusterNumer][selectedNodelNumber]].Clone();
    //  }

    //}

    public override void ShakeLocalParameters(IRandom random, double shakingFactor) {
      base.ShakeLocalParameters(random, shakingFactor);
      List<ISymbolicExpressionTreeNode> parametricNodes = new List<ISymbolicExpressionTreeNode>();
      Tree.Root.ForEachNodePostfix(n => {
        if (n.HasLocalParameters) parametricNodes.Add(n);
      });
      if (parametricNodes.Count > 0) {
        var selectedPoint = parametricNodes.SampleRandom(random);
        selectedPoint.ShakeLocalParameters(random, shakingFactor);
      }
    }

    public override IDeepCloneable Clone(Cloner cloner) {
      return new TreeModelTreeNode(this, cloner);
    }

    public override string ToString() {

      string s = "model" +" " +TreeNumber.ToString()+"\n " + new InfixExpressionFormatter().Format(tree);
      return s;
    }
  }
}


